<!DOCTYPE html>
<html>
<head>
  <meta charset="utf-8">
  <title>{% block title %}QUnit Test Runner{% endblock %}</title>

  {% load js %}

  {% block js_init %}
    {% css "test/js/libs/qunit.css" %}

    {% if use_django_js %}
    {% django_js %}
    {% endif %}
  {% endblock %}


</head>
<body>

    {% block body_content %}{% endblock %}

    <div id="qunit"></div>
    <div id="qunit-fixture"></div>

    {% js "test/js/libs/qunit.js" %}
    {% js "test/js/libs/qunit-tap.js" %}
    <script type="text/javascript">
        if (navigator.userAgent.indexOf('PhantomJS') != -1) {
            var logger = function() { console.log.apply(console, arguments); };
            qunitTap(QUnit, logger, {
              noPlan: false,
              showTestNameOnFailure: false,
              showModuleNameOnFailure:false
            });
        }
    </script>

    {% for file in js_test_files %}
    {% js file %}
    {% endfor %}

    {% block js_content %}{% endblock %}
</body>
</html>

