{% load js %}
<script type="text/javascript" src="{% url js_catalog %}"></script>
<script>
    window.DJANGO_JS = {
        URLS_JSON: '{% url django_js_urls %}',
        CONTEXT_JSON: '{% url django_js_context %}'
    };
</script>

{% if djangojs_jquery %}{% jquery_js %}{% endif %}
{% js "js/djangojs/django.js" %}

{% if djangojs_init %}
<script>
    Django.init();
</script>
{% endif %}
