{% extends "djangojs/jasmine-runner.html" %}

{% load js %}

{% block js_content %}
{% include "djangojs/test/test-data.html" %}
{% django_js init=false %}
{% js "test/js/specs/django.specs.js" %}
{% endblock %}

{% block body_content %}
  <form id="test-form" action="{% url test_form %}" method="POST" style="display: none;">
    {{csrf_token}}
    {{form}}
  </form>
{% endblock %}
